package ru.dbotthepony.mc.otm.datagen.loot

import net.minecraft.resources.ResourceLocation
import net.minecraft.util.valueproviders.UniformInt
import net.minecraft.world.entity.EntityType
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Rarity
import net.minecraft.world.level.storage.loot.BuiltInLootTables
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraft.world.level.storage.loot.predicates.LootItemKilledByPlayerCondition
import net.minecraftforge.common.loot.LootTableIdCondition
import ru.dbotthepony.mc.otm.data.ChanceWithPlaytimeCondition
import ru.dbotthepony.mc.otm.data.HasExosuitCondition
import ru.dbotthepony.mc.otm.data.IRandomizableItem
import ru.dbotthepony.mc.otm.data.ItemInInventoryCondition
import ru.dbotthepony.mc.otm.data.KilledByRealPlayer
import ru.dbotthepony.mc.otm.data.LootTableBasicAppender
import ru.dbotthepony.mc.otm.data.LootTableSeparatedAppender
import ru.dbotthepony.mc.otm.data.RandomizableItemLootAppender
import ru.dbotthepony.mc.otm.item.ProceduralExoSuitSlotUpgradeItem
import ru.dbotthepony.mc.otm.registry.MItems

fun LootTableIdCondition(location: String): LootItemCondition {
	return LootTableIdCondition.Builder(ResourceLocation("minecraft", location)).build()
}

fun LootTableIdCondition(location: ResourceLocation): LootItemCondition {
	return LootTableIdCondition.Builder(location).build()
}

fun addLootModifiers(it: LootModifiers) {
	it.add("dungeon_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SIMPLE_DUNGEON)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 2) to 0.5,
		ItemStack(MItems.PILL_HEAL, 1) to 0.75,
	))

	it.add("dungeon_exosuit", RandomizableItemLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SIMPLE_DUNGEON)),
		listOf(
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.2),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.05, rarity = Rarity.UNCOMMON)
		)
	))

	it.add("mineshaft_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.075,
		ItemStack(MItems.PILL_HEAL, 2) to 0.1,
		ItemStack(MItems.PILL_HEAL, 1) to 0.4,
	))

	it.add("mineshaft_exosuit", RandomizableItemLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		listOf(
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.1),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.01, rarity = Rarity.UNCOMMON)
		)
	))

	it.add("mineshaft_nutrient_paste", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.ABANDONED_MINESHAFT)),
		ItemStack(MItems.NUTRIENT_PASTE, 6) to 0.5,
		ItemStack(MItems.NUTRIENT_PASTE, 8) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.15,
	))

	it.add("desert_pyramid_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.DESERT_PYRAMID)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.05,
		ItemStack(MItems.PILL_HEAL, 1) to 0.3,
	))

	it.add("desert_pyramid_exosuit", RandomizableItemLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.DESERT_PYRAMID)),
		listOf(
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.15, rarity = Rarity.UNCOMMON),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.25, rarity = Rarity.COMMON),
		)
	))

	it.add("jungle_temple_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.JUNGLE_TEMPLE)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.5
	))

	it.add("jungle_temple_exosuit", RandomizableItemLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.JUNGLE_TEMPLE)),
		listOf(
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.15, rarity = Rarity.UNCOMMON),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.35, rarity = Rarity.RARE)
		)
	))

	it.add("end_city_modifications", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.END_CITY_TREASURE)),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
		ItemStack(MItems.PILL_HUMANE, 1) to 0.3,
		ItemStack(MItems.PILL_OBLIVION, 1) to 0.5,
		ItemStack(MItems.ZPM_BATTERY, 1) to 0.005,
	))

	it.add("end_city_exosuit", RandomizableItemLootAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.END_CITY_TREASURE)),
		listOf(
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.4, rarity = Rarity.UNCOMMON),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.2, rarity = Rarity.RARE),
			RandomizableItemLootAppender.Entry(MItems.ExosuitUpgrades.INVENTORY_UPGRADE_PROCEDURAL as IRandomizableItem, chance = 0.15, rarity = Rarity.EPIC),
		)
	))

	it.add("shipwreck_supply_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SHIPWRECK_SUPPLY)),
		ItemStack(MItems.PILL_HUMANE, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
	))

	it.add("shipwreck_supply_nutrient_paste", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition(BuiltInLootTables.SHIPWRECK_SUPPLY)),
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.85,
		ItemStack(MItems.NUTRIENT_PASTE, 24) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 38) to 0.1,
	))

	it.add("exosuit_probe_enderman_drop", LootTableBasicAppender(
		arrayOf(
			LootTableIdCondition(EntityType.ENDERMAN.defaultLootTable),
			HasExosuitCondition.INVERTED,
			KilledByRealPlayer,
			ChanceWithPlaytimeCondition(
				minPlaytime = 20 * 60 * 10,
				maxPlaytime = 20 * 60 * 120,
				minProbability = 0.1,
				maxProbability = 0.5,
			),
			ItemInInventoryCondition(
				ItemStack(MItems.EXOSUIT_PROBE)
			).invert().build()
		),
		listOf(
			ItemStack(MItems.EXOSUIT_PROBE)
		)
	))
}
